# -*- coding: utf-8 -*-
#COPYRIGHT 2017 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The GeoAnalytics toolbox contains a set of powerful tools for
performing spatial analysis on big data. GeoAnalytics Tools are
powered by your ArcGIS GeoAnalytics Server. ArcGIS GeoAnalytics Server
distributes the analysis between multiple server nodes. By using
distributed processing, you can process large datasets in less time."""
__all__ = ['AggregatePoints', 'CalculateDensity', 'CalculateField',
           'CopyToDataStore', 'CreateBuffers', 'CreateSpaceTimeCube',
           'DetectIncidents', 'FindHotSpots', 'FindSimilarLocations',
           'JoinFeatures', 'ReconstructTracks', 'SummarizeAttributes',
           'SummarizeWithin']
__alias__ = 'geoanalytics'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Analyze Patterns toolset
@gptooldoc('CalculateDensity_geoanalytics', None)
def CalculateDensity(input_layer=None, output_name=None, bin_type=None, bin_size=None, weight=None, neighborhood_size=None, fields=None, area_unit_scale_factor=None, time_step_interval=None, time_step_repeat=None, time_step_reference=None, data_store=None):
    """CalculateDensity_geoanalytics(input_layer, output_name, bin_type, bin_size, weight, neighborhood_size, {fields;fields...}, {area_unit_scale_factor}, {time_step_interval}, {time_step_repeat}, {time_step_reference}, {data_store})

        Calculates a magnitude-per-unit area from point features that fall
        within a neighborhood around each cell.

     INPUTS:
      input_layer (Feature Set):
          The points that will be used to calculate the density.
      output_name (String):
          Name of the output feature service.
      bin_type (String):
          Bin shape that will be used in analysis.

          * SQUARE-Square bins. This is the default.

          * HEXAGON-Hexagonal bins.
      bin_size (Linear Unit):
          Determines the size of the bins used to aggregate input features. When
          generating bins for squares, the number and units specified determine
          the height and length of the square. For hexagons, the number and
          units specified determine the distance between parallel sides.
      weight (String):
          The weighting applied to the density function.

          * UNIFORM-A magnitude-per-area calculation where each bin is equally
          weighted. This is the default.

          * KERNEL-A magnitude-per-area calculation with a smoothing algorithm
          applied (kernel) that weights bins closer to the points more heavily.
      neighborhood_size (Linear Unit):
          The search radius applied to density calculations.
      fields {Field}:
          One or more fields denoting population values for each feature. The
          population field is the count or quantity to be spread across the
          landscape to create a continuous surface.Values in the population
          field must be numeric. By default, the
          density of the count of input points will always be calculated.
      area_unit_scale_factor {String}:
          The desired area units of the output density values. The default
          unit is based on the units of the output spatial reference.

          * ACRES-Area in acres

          * HECTARES-Area in hectares

          * SQUARE_MILES-Area in square miles

          * SQUARE_KILOMETERS-Area in square kilometers

          * SQUARE_METERS-Area in square meters

          * SQUARE_FEET-Area in square feet

          * SQUARE_YARDS-Area in square yards
      time_step_interval {Time unit}:
          A value that specifies duration of the time step. This option is only
          available if the input points are time enabled and represent an
          instant in time. Time stepping can only be applied if time is enabled
          on your input.
          For more information on how time stepping works, see How time stepping
          works with GeoAnalytics Tools.
      time_step_repeat {Time unit}:
          A value that specifies how often the time-step interval occurs. This
          option is only available if the input points are time enabled and
          represent an instant in time.
      time_step_reference {Date}:
          A date that specifies the reference time with which to align the time
          steps. The default is January 1, 1970, at 12:00 a.m. This option is
          only available if the input points are time enabled and represent an
          instant in time.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CalculateDensity_geoanalytics(*gp_fixargs((input_layer, output_name, bin_type, bin_size, weight, neighborhood_size, fields, area_unit_scale_factor, time_step_interval, time_step_repeat, time_step_reference, data_store), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CreateSpaceTimeCube_geoanalytics', None)
def CreateSpaceTimeCube(point_layer=None, output_name=None, distance_interval=None, time_step_interval=None, time_step_interval_alignment=None, reference_time=None, summary_fields=None):
    """CreateSpaceTimeCube_geoanalytics(point_layer, output_name, distance_interval, time_step_interval, {time_step_interval_alignment}, {reference_time}, {summary_fields;summary_fields...})

        Summarizes a set of points into a netCDF data structure by
        aggregating them into space-time bins. Within each bin, the points are
        counted and specified attributes are aggregated. For all bin
        locations, the trend for counts and summary field values are
        evaluated.

     INPUTS:
      point_layer (Feature Set):
          The input point feature class to be aggregated into space-time bins.
      output_name (String):
          The output netCDF data cube that will be created to contain counts
          and summaries of the input feature point data.
      distance_interval (Linear Unit):
          The size of the bins used to aggregate the point_layer. All points
          that fall within the same distance_interval and time_interval will be
          aggregated. The distance that will determine the bin size.
      time_step_interval (Time unit):
          The number of seconds,  minutes, hours, days, weeks, or years that
          will represent a single time step. All points within the same
          time_interval and distance_interval will be aggregated. Examples of
          valid entries for this parameter are 1 Weeks, 13 Days, or 1 Months.
      time_step_interval_alignment {String}:
          Defines how aggregation will occur based on a given Time Step
          Interval.

          * END_TIME-Time steps align to the last time event and aggregate back
          in time.

          * START_TIME-Time steps align to the first time event and aggregate
          forward in time.

          * REFERENCE_TIME-Time steps align to a particular date/time that you
          specify. If all points in the input features have a timestamp larger
          than the Reference time you provide (or it falls exactly on the start
          time of the input features), the time-step interval will begin with
          that reference time and aggregate forward in time (as occurs with a
          Start time alignment). If all points in the input features have a
          timestamp smaller than the reference time you provide (or it falls
          exactly on the end time of the input features), the time-step interval
          will end with that reference time and aggregate backward in time (as
          occurs with an End time alignment). If the Reference time you provide
          is in the middle of the time extent of your data, a time-step interval
          will be created ending with the reference time provided (as occurs
          with an End time alignment); additional intervals will be created both
          before and after the reference time until the full time extent of your
          data is covered.
      reference_time {Date}:
          The date/time to use to align the time-step intervals. If you want to
          bin your data weekly from Monday to Sunday, for example, you could set
          a reference time of Sunday at midnight to ensure bins break between
          Sunday and Monday at midnight.
      summary_fields {Value Table}:
          The numeric field containing attribute values used to calculate the
          specified statistic when aggregating into a space time cube. Multiple
          statistic and field combinations may be specified. Null values are
          excluded from all statistical calculations.Available statistic types
          are the following:

          * SUM-Adds the total value for the specified field within each bin.

          * MEAN-Calculates the average for the specified field within each bin.

          * MIN-Finds the smallest value for all records of the specified field
          within each bin.

          * MAX-Finds the largest value for all records of the specified field
          within each bin.

          * STDDEV-Finds the standard deviation on values in the specified field
          within each bin.
          Available fill types are the following:

          * ZEROS-Fills empty bins with zeros.

          * SPATIAL_NEIGHBORS-Fills empty bins with the average value of spatial
          neighbors.

          * SPACE_TIME_NEIGHBORS-Fills empty bins with the average value of
          space-time neighbors.

          * TEMPORAL_TREND-Fills empty bins using an interpolated univariate
          spline algorithm.
          Null values present in any of the summary fields will result in those
          features being excluded from the analysis. If having the count of
          points in each bin is part of your analysis strategy, you may want to
          consider creating separate cubes, one for the count (without summary
          fields) and one for summary fields. If the set of null values is
          different for each summary field, you may also consider creating a
          separate cube for each summary field."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateSpaceTimeCube_geoanalytics(*gp_fixargs((point_layer, output_name, distance_interval, time_step_interval, time_step_interval_alignment, reference_time, summary_fields), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('FindHotSpots_geoanalytics', None)
def FindHotSpots(point_layer=None, output_name=None, bin_size=None, neighborhood_size=None, time_step_interval=None, time_step_alignment=None, time_step_reference=None, data_store=None):
    """FindHotSpots_geoanalytics(point_layer, output_name, bin_size, neighborhood_size, {time_step_interval}, {time_step_alignment}, {time_step_reference}, {data_store})

        Given a set of features, identifies statistically significant hot
        spots and cold spots using the Getis-Ord Gi* statistic.

     INPUTS:
      point_layer (Feature Set):
          The point feature class for which hot spot analysis will be performed.
      output_name (String):
          The output feature class to receive the z-score and p-value results.
      bin_size (Linear Unit):
          The distance interval that represents the bin size and units into
          which the point_layer will be aggregated. The distance interval must
          be a linear unit.
      neighborhood_size (Linear Unit):
          The spatial extent of the analysis neighborhood. This value
          determines which features are analyzed together in order to assess
          local clustering.
      time_step_interval {Time unit}:
          Only used if time is enabled on your point_layer. The interval for
          time step.
      time_step_alignment {String}:
          How you want your time steps aligned. This option is only available
          if the input points are time enabled and represent an instant in time.

          * END_TIME-Time steps align to the last time event and aggregate back
          in time.

          * START_TIME-Time steps align to the first time event and aggregate
          forward in time.

          * REFERENCE_TIME-Time steps align to a particular date/time that you
          specify. If all points in the input features have a timestamp larger
          than the Reference time you provide (or it falls exactly on the start
          time of the input features), the time-step interval will begin with
          that reference time and aggregate forward in time (as occurs with a
          Start time alignment). If all points in the input features have a
          timestamp smaller than the reference time you provide (or it falls
          exactly on the end time of the input features), the time-step interval
          will end with that reference time and aggregate backward in time (as
          occurs with an End time alignment). If the Reference time you provide
          is in the middle of the time extent of your data, a time-step interval
          will be created ending with the reference time provided (as occurs
          with an End time alignment); additional intervals will be created both
          before and after the reference time until the full time extent of your
          data is covered.
      time_step_reference {Date}:
          The time you want your time steps and time intervals to align. Only
          used if time is enabled on your point_layer.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FindHotSpots_geoanalytics(*gp_fixargs((point_layer, output_name, bin_size, neighborhood_size, time_step_interval, time_step_alignment, time_step_reference, data_store), True)))
        return retval
    except Exception as e:
        raise e


# Find Locations toolset
@gptooldoc('DetectIncidents_geoanalytics', None)
def DetectIncidents(input_layer=None, output_name=None, track_fields=None, start_condition=None, end_condition=None, output_mode=None, data_store=None):
    """DetectIncidents_geoanalytics(input_layer, output_name, track_fields;track_fields..., start_condition, {end_condition}, {output_mode}, {data_store})

        Creates a new layer that detects features that meet a given condition.

     INPUTS:
      input_layer (Record Set):
          Input features that contain potential incidents.
      output_name (String):
          Name of the output feature service.
      track_fields (Field):
          One or more fields used to identify unique tracks.
      start_condition (Calculator Expression):
          The condition used to identify incidents. Expressions are written in
          Arcade and can include [+ - * / ] operators and multiple fields.If
          your layer is added to the map, the Fields and Helper filters can
          be used to build an expression.
      end_condition {Calculator Expression}:
          The condition used to end incidents. If no end condition is specified,
          incidents will end when the start condition is no longer
          true.Expressions are written in Arcade and can include [+ - * / ]
          operators
          and multiple fields.If your layer is added to the map, the Fields and
          Helper filters can
          be used to build an expression.
      output_mode {String}:
          Specifies the features to return, all features or only incidents.

          * ALL_FEATURES-All of the input features are returned. This is the
          default.

          * INCIDENTS-Only features that were found to be incidents are
          returned.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.DetectIncidents_geoanalytics(*gp_fixargs((input_layer, output_name, track_fields, start_condition, end_condition, output_mode, data_store), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('FindSimilarLocations_geoanalytics', None)
def FindSimilarLocations(input_layer=None, search_layer=None, output_name=None, analysis_fields=None, most_or_least_similar=None, match_method=None, number_of_results=None, append_fields=None, data_store=None):
    """FindSimilarLocations_geoanalytics(input_layer, search_layer, output_name, analysis_fields;analysis_fields..., {most_or_least_similar}, {match_method}, {number_of_results}, {append_fields;append_fields...}, {data_store})

        Identifies which candidate features are most similar or most
        dissimilar to one or more input features based on feature attributes.

     INPUTS:
      input_layer (Record Set):
          The reference layer (or a selection on a layer) containing the
          features you want to match; you are searching for other features that
          look like these features. When more than one feature is provided,
          matching is based on attribute averages.
      search_layer (Record Set):
          The candidate layer (or a selection on a layer) containing candidate
          matching features. The tool will look for features most like (or most
          dislike) the input_layer among these candidates.
      output_name (String):
          The output feature service contains a record for each of the
          input_layer and for all the solution-matching features found.
      analysis_fields (String):
          A list of numeric attributes representing the matching criteria.
      most_or_least_similar {String}:
          Choose whether you are interested in features that are most alike or
          most different to the input_layer.

          * MOST_SIMILAR-Find the features that are most alike.

          * LEAST_SIMILAR-Find the features that are most different.

          * BOTH-Find both the features that are most alike and the features
          that are most different.
      match_method {String}:
          Choose whether matching should be based on values or cosine
          relationships.

          * ATTRIBUTE_VALUES-Similarity or dissimilarity will be based on the
          sum of squared standardized attribute value differences for all the
          analysis_fields.

          * ATTRIBUTE_PROFILES-Similarity or dissimilarity will be computed as a
          function of cosine similarity for all the analysis_fields.
      number_of_results {Long}:
          The number of solution matches to find. Entering zero or a number
          larger than the total number of  search_layer will return rankings for
          all the candidate features, with a maximum of 10,000.
      append_fields {Field}:
          An optional list of attributes to include with the output_name. You
          might want to include a name identifier, categorical field, or date
          field, for example. These fields are not used to determine similarity;
          they are only included in the output_name for your reference. By
          default, all fields are added.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.FindSimilarLocations_geoanalytics(*gp_fixargs((input_layer, search_layer, output_name, analysis_fields, most_or_least_similar, match_method, number_of_results, append_fields, data_store), True)))
        return retval
    except Exception as e:
        raise e


# Manage Data toolset
@gptooldoc('CalculateField_geoanalytics', None)
def CalculateField(input_layer=None, output_name=None, field_name=None, field_type=None, expression=None, track_aware=None, track_fields=None, data_store=None):
    """CalculateField_geoanalytics(input_layer, output_name, field_name, field_type, expression, {track_aware}, {track_fields;track_fields...}, {data_store})

        Creates a new layer with calculated field values.

     INPUTS:
      input_layer (Record Set):
          Input features that will have a field calculated.
      output_name (String):
          Name of the output feature service.
      field_name (String):
          Field that will have values calculated. This can be an existing field
          or a new field name.
      field_type (String):
          The field type for the calculated field.

          * STRING-The new field will be of type text.

          * INTEGER-The new field will be of type integer.

          * FLOAT-The new field will be of type float.

          * DATE-The new field will be of type date.
      expression (Calculator Expression):
          The expression used to calculate values in your field. Expressions are
          written in Arcade and can include operators and multiple fields.
          Calculated values are applied in the units of the spatial reference of
          the input, unless you are using a geographic coordinate system, in
          which case they will be in meters.If your layer is added to the map,
          the Fields and Helper filters can
          be used to build an expression.
      track_aware {Boolean}:
          Specifies whether the expression uses a track-aware expression.

          * TRACK_AWARE-The expression uses track-aware expressions, and a track
          field must be specified.

          * NOT_TRACK_AWARE-The expression does not use track-aware expressions.
          This is the default.
      track_fields {Field}:
          One or more fields used to identify unique tracks.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CalculateField_geoanalytics(*gp_fixargs((input_layer, output_name, field_name, field_type, expression, track_aware, track_fields, data_store), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('CopyToDataStore_geoanalytics', None)
def CopyToDataStore(input_layer=None, output_name=None, data_store=None):
    """CopyToDataStore_geoanalytics(input_layer, output_name, {data_store})

        Copies features from the input to a new feature service in your
        portal.

     INPUTS:
      input_layer (Record Set):
          The features to be copied.
      output_name (String):
          Name of output feature service.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CopyToDataStore_geoanalytics(*gp_fixargs((input_layer, output_name, data_store), True)))
        return retval
    except Exception as e:
        raise e


# Summarize Data toolset
@gptooldoc('AggregatePoints_geoanalytics', None)
def AggregatePoints(point_layer=None, output_name=None, polygon_or_bin=None, polygon_layer=None, bin_type=None, bin_size=None, time_step_interval=None, time_step_repeat=None, time_step_reference=None, summary_fields=None, data_store=None):
    """AggregatePoints_geoanalytics(point_layer, output_name, polygon_or_bin, {polygon_layer}, {bin_type}, {bin_size}, {time_step_interval}, {time_step_repeat}, {time_step_reference}, {summary_fields;summary_fields...}, {data_store})

        Aggregates points into polygon features or bins. At all locations
        where points exist, a polygon is returned with a count of points as
        well as optional statistics.

     INPUTS:
      point_layer (Feature Set):
          The point features to be aggregated into polygons or bins.
      output_name (String):
          Name of the output feature service.
      polygon_or_bin (String):
          What the point_layer will be aggregated into.

          * POLYGONS-The points layer will be aggregated into a polygons
          dataset.

          * BINS-The points will be aggregated into square or hexagonal bins
          that are generated when the tool is run.
      polygon_layer {Feature Set}:
          The polygon features into which the input points will be aggregated.
      bin_type {String}:
          Bin shape that will be generated to aggregate points into.

          * SQUARE-Square bins where the bin_size represents the height of a
          square. This is the default.

          * HEXAGON-Hexagonal bins where the bin_size represents the height
          between two parallel sides.
      bin_size {Linear Unit}:
          The distance interval that represents the bin_size and units into
          which the point_layer will be aggregated. The distance interval must
          be a linear unit.
      time_step_interval {Time unit}:
          A value that specifies duration of the time step. This option is only
          available if the input points are time enabled and represent an
          instant in time. Time stepping can only be applied if time is enabled
          on your input.
          For more information on how time stepping works, see How time stepping
          works with GeoAnalytics Tools.
      time_step_repeat {Time unit}:
          A value that specifies how often the time-step interval occurs. This
          option is only available if the input points are time enabled and
          represent an instant in time.
      time_step_reference {Date}:
          A date that specifies the reference time with which to align the time
          steps. The default is January 1, 1970, at 12:00 a.m. This option is
          only available if the input points are time enabled and represent an
          instant in time.
      summary_fields {Value Table}:
          Statistics that will be calculated on specified fields.

          * COUNT-Calculates the number of nonnull values. Can be used on
          numeric fields or strings. The count of [null, 0, 2] is 2.

          * SUM-The sum of numeric values in a field. The sum of [null, null, 3]
          is 3.

          * MEAN-The mean of numeric values. The mean of [0, 2, null] is 1.

          * MIN-The minimum value of a numeric field. The minimum of [0, 2,
          null] is 0.

          * MAX-The maximum value of a numeric field. The maximum value of [0,
          2, null] is 2.

          * STDDEV-The standard deviation of a numeric field. The standard
          deviation of [1] is null. The standard deviation of [null, 1,1,1] is
          1.

          * VAR-The variance of a numeric field in a track. The variance of [1]
          is null. The variance of [null, 1, 1, 1] is 1.

          * RANGE-The range of a numeric field. This is calculated as the
          minimum values subtracted form the maximum value. The range of [0,
          null, 1] is 1. The range of [null, 4] is 0.

          * ANY-This is a sample string from a field of type string.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.AggregatePoints_geoanalytics(*gp_fixargs((point_layer, output_name, polygon_or_bin, polygon_layer, bin_type, bin_size, time_step_interval, time_step_repeat, time_step_reference, summary_fields, data_store), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('JoinFeatures_geoanalytics', None)
def JoinFeatures(target_layer=None, join_layer=None, output_name=None, join_operation=None, spatial_relationship=None, spatial_near_distance=None, temporal_relationship=None, temporal_near_distance=None, attribute_relationship=None, summary_fields=None, join_condition=None, data_store=None):
    """JoinFeatures_geoanalytics(target_layer, join_layer, output_name, join_operation, {spatial_relationship}, {spatial_near_distance}, {temporal_relationship}, {temporal_near_distance}, {attribute_relationship;attribute_relationship...}, {summary_fields;summary_fields...}, {join_condition}, {data_store})

        Joins attributes from one layer to another based on spatial, temporal,
        and attribute relationships, or a combination of those relationships.

     INPUTS:
      target_layer (Record Set):
          Attributes of the target features and the attributes from the joined
          features are transferred to the output.
      join_layer (Record Set):
          The attributes from the join features are joined to the attributes of
          the target features. See the explanation of the Join Operation
          (join_operation for Python) parameter for details on how the
          aggregation of joined attributes are affected by the type of join
          operation.
      output_name (String):
          Name of the output feature service.
      join_operation (String):
          Determines how joins between the target_layer and join_layer will be
          handled in the output feature if multiple join features are found that
          have the same spatial relationship with a single target feature.

          * JOIN_ONE_TO_ONE-If multiple join features are found that have the
          same relationship with a single target feature, the attributes from
          the multiple join features will be aggregated. For example, if a point
          target feature is found within two separate polygon join features, the
          attributes from the two polygons will be aggregated before being
          transferred to the output point feature class. If one polygon has an
          attribute value of 3 and the other has a value of 7, and the summary
          statistic sum is specified for that field, the aggregated value in the
          output feature class will be 10. The default is that only the count
          statistics is returned. Join one to one is the default.

          * JOIN_ONE_TO_MANY-If multiple join features are found that have the
          same relationship with a single target feature, the output feature
          class will contain multiple copies (records) of the target feature.
          For example, if a single-point target feature is found within two
          separate polygon join features, the output feature class will contain
          two copies of the target feature: one record with the attributes of
          one polygon and another record with the attributes of the other
          polygon. There are no summary statistics available with this method.
      spatial_relationship {String}:
          Defines the criteria used to spatially join features.

          * INTERSECTS-The features in the join features will be matched if they
          intersect a target feature. This is the default.

          * EQUALS-The features in the join features will be matched if they are
          the same geometry as a target feature.

          * NEAR-The features in the join features will be matched if they are
          within a specified distance of a target feature. Specify a distance in
          the spatial near distance parameter.

          * CONTAINS-The features in the join features will be matched if a
          target feature contains them. The target features must be polygons or
          polylines. The join features can only be polygons when the target
          features are also polygons. Polygon can contain any feature type.
          Polyline can contain only polyline and point. Point cannot contain any
          feature, not even a point. If the join feature is entirely on the
          boundary of the target feature (no part is properly inside or
          outside), the feature will not be matched.

          * WITHIN-The features in the join features will be matched if a target
          feature is within them. It is opposite to the contains relationship.
          For this option, the target features can only be polygons when the
          join features are also polygons. Point can be join feature only if
          point is target. If the entirety of the feature in the join features
          is on the boundary of the target feature, the feature will not be
          matched.

          * TOUCHES-The features in the join features will be matched if they
          have a boundary that touches a target feature. When the target and
          join features are lines or polygons, the boundary of the join feature
          can only touch the boundary of the target feature and no part of the
          join feature can cross the boundary of the target feature.

          * CROSSES-The features in the join features will be matched if a
          target feature is crossed by their outline. The join and target
          features must be lines or polygons. If polygons are used for the join
          or target features, the polygon's boundary (line) will be used. Lines
          that cross at a point will be matched, not lines that share a line
          segment.

          * OVERLAPS-The features in the join features will be matched if they
          overlap a target feature.
      spatial_near_distance {Linear Unit}:
          Join features within this distance of a target feature will be
          considered for the spatial join. A search radius is only valid when
          the spatial_relationship is NEAR.
      temporal_relationship {String}:
          Defines the time criteria used to match features.

          * MEETS-When a target time interval end is equal to the join time
          interval start, the target time meets the join time.

          * MET_BY-When a target time interval start is equal to the join time
          interval end, the target time is met by the join time.

          * OVERLAPS-When a target time interval starts and ends before the
          start and end of the join time interval, the target time overlaps the
          join time.

          * OVERLAPPED_BY-When a target time interval starts and ends after the
          start and end time of the join time interval, the target time is
          overlapped by the join time.

          * DURING-When a target time occurs between the start and end of the
          join time interval, the target time is during the join time.

          * CONTAINS-When a join feature time occurs between the start and end
          of the target time interval, the target time contains the join time.

          * EQUALS-Two times are considered equal if their instants or intervals
          are identical.

          * FINISHES-When a target time ends at the same time as a join time,
          and the target time started after the join time, the target time
          finishes the join time.

          * FINISHED_BY-When a join feature time ends at the same time as a
          target time, and the join time started after the target time, the
          target time is finished by the join time.

          * STARTS-When a target time starts at the same time as the join time
          interval start and ends before the join time interval ends, the target
          time starts the join time.

          * STARTED_BY-When a target interval time starts at the same time as
          the join time and ends after the join time, the target time is started
          by the join time.

          * INTERSECTS-When any part of a target time occurs at the same time as
          the join time, the target time intersects the join time.

          * NEAR-When a target time is within a specified range of time from the
          join time, the target time is near the join time.
      temporal_near_distance {Time unit}:
          Join features within this distance in time of a target feature will be
          considered for the spatial join. A time is only valid when the
          temporal_relationship is NEAR and both feature are time enabled.
      attribute_relationship {Value Table}:
          Join features based on values within an attribute field. You must
          specify which attribute field from the target layer matches an
          attribute field from the join layer.

          * Target Field-An attribute field from the target layer containing
          values to match.

          * Join Field-An attribute field from the join layer containing values
          to match.
      summary_fields {Value Table}:
          Statistics that will be calculated on specified fields.

          * COUNT-Calculates the number of nonnull values. Can be used on
          numeric fields or strings. The count of [null, 0, 2] is 2.

          * SUM-The sum of numeric values in a field. The sum of [null, null, 3]
          is 3.

          * MEAN-The mean of numeric values. The mean of [0, 2, null] is 1.

          * MIN-The minimum value of a numeric field. The minimum of [0, 2,
          null] is 0.

          * MAX-The maximum value of a numeric field. The maximum value of [0,
          2, null] is 2.

          * STDDEV-The standard deviation of a numeric field. The standard
          deviation of [1] is null. The standard deviation of [null, 1,1,1] is
          1.

          * VAR-The variance of a numeric field in a track. The variance of [1]
          is null. The variance of [null, 1, 1, 1] is 1.

          * RANGE-The range of a numeric field. This is calculated as the
          minimum values subtracted form the maximum value. The range of [0,
          null, 1] is 1. The range of [null, 4] is 0.

          * ANY-This is a sample string from a field of type string.
      join_condition {String}:
          Applies a condition to specified fields. Only features with fields
          that meet these conditions will be joined.For example, you could apply
          a join to features where the attribute
          HealthSpending in the join layer is greater than 20 percent of the
          attribute Income in the target layer. At 10.5 and 10.5.1 the join
          condition to apply this expression is join["HealthSpending"] >
          target["Income"] * .2. At 10.6 and above, use Arcade Expressions such
          as $join["HealthSpending"] > $target["Income"] * .2
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.JoinFeatures_geoanalytics(*gp_fixargs((target_layer, join_layer, output_name, join_operation, spatial_relationship, spatial_near_distance, temporal_relationship, temporal_near_distance, attribute_relationship, summary_fields, join_condition, data_store), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('ReconstructTracks_geoanalytics', None)
def ReconstructTracks(input_layer=None, output_name=None, track_fields=None, method=None, buffer_type=None, buffer_field=None, buffer_expression=None, time_split=None, summary_fields=None, data_store=None, distance_split=None):
    """ReconstructTracks_geoanalytics(input_layer, output_name, track_fields;track_fields..., method, {buffer_type}, {buffer_field}, {buffer_expression}, {time_split}, {summary_fields;summary_fields...}, {data_store}, {distance_split})

        Creates tracks (either lines or polygons) from time-enabled input
        data.

     INPUTS:
      input_layer (Feature Set):
          Points or polygons to be reconstructed to tracks. The input must be a
          time-enabled layer that represent an instant in time.
      output_name (String):
          Name of the output feature service.
      track_fields (Field):
          One or more fields used to identify unique tracks.
      method (String):
          Defines the criteria used to reconstruct tracks. If a buffer is used,
          method determines the type of buffer.

          * GEODESIC-If GEODESIC is selected and the spatial reference is
          pannable, tracks will cross the dateline when appropriate. If GEODESIC
          is selected and the spatial reference is not pannable, tracks will be
          limited to the coordinate system extent and may not wrap.

          * PLANAR-If PLANAR is selected, the tracks will never cross the
          dateline.
      buffer_type {String}:
          How the buffer distance is defined.

          * FIELD-Uses a single field to define the buffer distance.

          * EXPRESSION-An equation using fields and mathematical operators.
      buffer_field {Field}:
          The  field used to buffer the input features. Field values are applied
          in the units of the spatial reference of the input unless you are
          using a geographic coordinate system, in which case, they will be in
          meters.
      buffer_expression {Calculator Expression}:
          The expression used to buffer input features. Fields must be numeric,
          and the expression can include [+ - * / ] operators and multiple
          fields. Calculated values are applied in the units of the spatial
          reference of the input unless you are using a geographic coordinate
          system, in which case they will be in meters. With ArcGIS Enterprise
          10.5 and 10.5.1, expressions will be formatted
          as the following:as_kilometers(distance) * 2 + as_meters(15). With
          ArcGIS Enterprise 10.6 or later, use Arcade expressions such as
          as_kilometers($feature.distance) * 2 + as_meters(15).
      time_split {Time unit}:
          Features further apart in time than the time-split duration will be
          split into separate tracks.
      summary_fields {Value Table}:
          Statistics that will be calculated on specified fields.

          * COUNT-Calculates the number of nonnull values. Can be used on
          numeric fields or strings. The count of [null, 0, 2] is 2.

          * SUM-The sum of numeric values in a field. The sum of [null, null, 3]
          is 3.

          * MEAN-The mean of numeric values. The mean of [0, 2, null] is 1.

          * MIN-The minimum value of a numeric field. The minimum of [0, 2,
          null] is 0.

          * MAX-The maximum value of a numeric field. The maximum value of [0,
          2, null] is 2.

          * STDDEV-The standard deviation of a numeric field. The standard
          deviation of [1] is null. The standard deviation of [null, 1,1,1] is
          1.

          * VAR-The variance of a numeric field in a track. The variance of [1]
          is null. The variance of [null, 1, 1, 1] is 1.

          * RANGE-The range of a numeric field. This is calculated as the
          minimum values subtracted form the maximum value. The range of [0,
          null, 1] is 1. The range of [null, 4] is 0.

          * ANY-This is a sample string from a field of type string.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store.
      distance_split {Linear Unit}:
          Features further apart in distance than the distance split value will
          be split into separate tracks. This parameter is only available with
          ArcGIS Enterprise 10.6 and above."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ReconstructTracks_geoanalytics(*gp_fixargs((input_layer, output_name, track_fields, method, buffer_type, buffer_field, buffer_expression, time_split, summary_fields, data_store, distance_split), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('SummarizeAttributes_geoanalytics', None)
def SummarizeAttributes(input_layer=None, output_name=None, fields=None, summary_fields=None, data_store=None):
    """SummarizeAttributes_geoanalytics(input_layer, output_name, fields;fields..., {summary_fields;summary_fields...}, {data_store})

        Calculates summary statistics for fields in a feature class.

     INPUTS:
      input_layer (Record Set):
          The point, polyline, or polygon layer to be summarized.
      output_name (String):
          Name of the output feature service.
      fields (Field):
          One or more fields used to summarize similar features. For example,
          if you choose a single field called PropertyType with the values of
          commercial and residential, all of the residential fields would be
          summarized together, with summary statistics calculated, and all of
          the commercial fields would be summarized together.
      summary_fields {Value Table}:
          Statistics that will be calculated on specified fields.

          * COUNT-Calculates the number of nonnull values. Can be used on
          numeric fields or strings. The count of [null, 0, 2] is 2.

          * SUM-The sum of numeric values in a field. The sum of [null, null, 3]
          is 3.

          * MEAN-The mean of numeric values. The mean of [0, 2, null] is 1.

          * MIN-The minimum value of a numeric field. The minimum of [0, 2,
          null] is 0.

          * MAX-The maximum value of a numeric field. The maximum value of [0,
          2, null] is 2.

          * STDDEV-The standard deviation of a numeric field. The standard
          deviation of [1] is null. The standard deviation of [null, 1,1,1] is
          1.

          * VAR-The variance of a numeric field in a track. The variance of [1]
          is null. The variance of [null, 1, 1, 1] is 1.

          * RANGE-The range of a numeric field. This is calculated as the
          minimum values subtracted form the maximum value. The range of [0,
          null, 1] is 1. The range of [null, 4] is 0.

          * ANY-This is a sample string from a field of type string.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SummarizeAttributes_geoanalytics(*gp_fixargs((input_layer, output_name, fields, summary_fields, data_store), True)))
        return retval
    except Exception as e:
        raise e

@gptooldoc('SummarizeWithin_geoanalytics', None)
def SummarizeWithin(summarized_layer=None, output_name=None, polygon_or_bin=None, bin_type=None, bin_size=None, summary_polygons=None, sum_shape=None, shape_units=None, standard_summary_fields=None, weighted_summary_fields=None, data_store=None):
    """SummarizeWithin_geoanalytics(summarized_layer, output_name, {polygon_or_bin}, {bin_type}, {bin_size}, {summary_polygons}, {sum_shape}, {shape_units}, {standard_summary_fields;standard_summary_fields...}, {weighted_summary_fields;weighted_summary_fields...}, {data_store})

        Overlays a polygon layer with another layer to summarize the number of
        points, length of the lines, or area of the polygons within each
        polygon, and calculates attribute field statistics about those
        features within the polygons.The following are examples of scenarios
        using Summarize Within:

        * Given watershed boundaries and land-use boundaries by land-use type,
        calculate total acreage of land-use type for each watershed.

        * Given county parcels and city boundaries, summarize the average
        value of vacant parcels within each city boundary.

        * Given counties and roads, summarize the total mileage of roads by
        road type within each county.

     INPUTS:
      summarized_layer (Feature Set):
          The point, line, or polygon features that will be summarized by
          either polygons or bins.
      output_name (String):
          The name of the output polygon feature service containing the
          intersecting geometries and attributes.
      polygon_or_bin {String}:
          Specify whether the summarized_layer will be summarized by polygons or
          bins.

          * POLYGONS-The summarized layer will be aggregated into a polygon
          dataset.

          * BINS-The summarized layer will be aggregated into square or
          hexagonal bins.
      bin_type {String}:
          Bin shape that will be generated to summarize features into.

          * SQUARE-bin_size represents the height of a square. This is the
          default.

          * HEXAGON-bin_size represents the height between two parallel sides.
      bin_size {Linear Unit}:
          The distance interval that represents the bin size and units by which
          the input features will be summarized.
      summary_polygons {Feature Set}:
          The polygons used to summarize the features in the input summarized
          layer.
      sum_shape {Boolean}:
          Calculate the length of lines or area of polygons within your summary
          layer (polygon or bin). The count of points, lines, and polygons
          intersecting your summary shape will always be included.

          * ADD_SUMMARY-Summary shape values will be calculated. This is the
          default.

          * NO_SUMMARY-Summary shape values will not be calculated.
      shape_units {String}:
          The unit in which to calculate shape summary attributes. If the input
          summary_layer is points, no shape unit is necessary, since only the
          count of points within each input polygon is added. If the input
          summary features are lines, specify a linear unit. If the input
          summary features are polygons, specify an areal unit.

          * METERS-Meters

          * KILOMETERS-Kilometers

          * FEET-Feet

          * YARDS-Yards

          * MILES-Miles

          * ACRES-Acres

          * HECTARES-Hectares

          * SQUARE_METERS-Square meters

          * SQUARE_KILOMETERS-Square kilometers

          * SQUARE_FEET-Square feet

          * SQUARE_YARDS-Square yards

          * SQUARE_MILES-Square miles
      standard_summary_fields {Value Table}:
          Statistics that will be calculated on specified fields.

          * COUNT-Calculates the number of nonnull values. Can be used on
          numeric fields or strings. The count of [null, 0, 2] is 2.

          * SUM-The sum of numeric values in a field. The sum of [null, null, 3]
          is 3.

          * MEAN-The mean of numeric values. The mean of [0, 2, null] is 1.

          * MIN-The minimum value of a numeric field. The minimum of [0, 2,
          null] is 0.

          * MAX-The maximum value of a numeric field. The maximum value of [0,
          2, null] is 2.

          * STDDEV-The standard deviation of a numeric field. The standard
          deviation of [1] is null. The standard deviation of [null, 1,1,1] is
          1.

          * VAR-The variance of a numeric field in a track. The variance of [1]
          is null. The variance of [null, 1, 1, 1] is 1.

          * RANGE-The range of a numeric field. This is calculated as the
          minimum values subtracted form the maximum value. The range of [0,
          null, 1] is 1. The range of [null, 4] is 0.

          * ANY-This is a sample string from a field of type string.
      weighted_summary_fields {Value Table}:
          Weighted statistics that will be calculated on specified fields.

          * COUNT-The count of each field multiplied by the proportion of the
          summarized layer within the polygons.

          * SUM-The sum of weighted of values in each field, where the weight
          applied is the proportion of the summarized layer within the polygons.

          * MEAN-The weighted mean of values in each field, where the weight
          applied is the proportion of the summarized layer within the polygons.

          * MIN-The minimum of weighted values in each field, where the weight
          applied is the proportion of the summarized layer within the polygons.

          * MAX-The maximum of weighted values in each field, where the weight
          applied is the proportion of the summarized layer within the polygons.

          * RANGE-Finds the difference between MIN and MAX.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.SummarizeWithin_geoanalytics(*gp_fixargs((summarized_layer, output_name, polygon_or_bin, bin_type, bin_size, summary_polygons, sum_shape, shape_units, standard_summary_fields, weighted_summary_fields, data_store), True)))
        return retval
    except Exception as e:
        raise e


# Use Proximity toolset
@gptooldoc('CreateBuffers_geoanalytics', None)
def CreateBuffers(input_layer=None, output_name=None, method=None, buffer_type=None, buffer_field=None, buffer_distance=None, buffer_expression=None, dissolve_option=None, dissolve_fields=None, summary_fields=None, multipart=None, data_store=None):
    """CreateBuffers_geoanalytics(input_layer, output_name, method, buffer_type, {buffer_field}, {buffer_distance}, {buffer_expression}, {dissolve_option}, {dissolve_fields;dissolve_fields...}, {summary_fields;summary_fields...}, {multipart}, {data_store})

        Creates buffers around input features to a specified distance.

     INPUTS:
      input_layer (Feature Set):
          The point, polyline, or polygon features to be buffered.
      output_name (String):
          Name of the output feature service.
      method (String):
          Specifies what method to use, planar or geodesic, to create the
          buffer.

          * GEODESIC-    Buffers are created using a shape-preserving geodesic
          buffer method regardless of input coordinate system. This is the
          default.

          * PLANAR-      If the input features are in a projected coordinate
          system, Euclidean buffers are created. If the input features are in a
          geographic coordinate system, geodesic buffers are created. The Output
          Coordinate System environment setting can be used to specify a
          coordinate system.
      buffer_type (String):
          Specifies how the buffer distance is defined.

          * DISTANCE-Apply the same linear distance for all features.

          * FIELD-Select a numeric or string field to represent the buffer
          distance.

          * EXPRESSION-Build an equation using fields, constants, and
          mathematical operations to represent the buffer distance.
      buffer_field {Field}:
          The field name that contains the buffer distance for each feature. If
          a field value is a number, it is assumed that the distance is in the
          linear unit of the input_layer spatial reference, unless the
          input_layer is in a geographic coordinate system, in which case, the
          value is assumed to be in meters. If the linear unit specified in the
          field values is invalid or not recognized, the linear unit of the
          input features' spatial reference will be used by default.
      buffer_distance {Linear Unit}:
          The distance around the input features that will be buffered.
          Distances can be expressed in meters, kilometers, feet, yards, miles,
          or nautical miles.
      buffer_expression {Calculator Expression}:
          An equation using fields and mathematical operators that will be
          applied as a buffer to each feature. Fields must be numeric and the
          expression can include [+ - * / ] operators and multiple fields.
          Calculated values are applied in meters unless otherwise specified.
          For example, to apply a buffer that multiples a numeric field named
          distance in kilometers by 2 and adds 15 meters. With ArcGIS Enterprise
          10.5 and 10.5.1 expressions will be formatted
          as the following:as_kilometers(distance) * 2 + as_meters(15). With
          ArcGIS Enterprise 10.6 or later, use Arcade expressions such as
          as_kilometers($feature["distance"]) * 2 + as_meters(15).
      dissolve_option {String}:
          Specifies the dissolve option to be performed to remove buffer
          overlap.

          * NONE-An individual buffer for each feature is maintained regardless
          of overlap. This is the default.

          * ALL-All buffers are dissolved together into a single feature,
          removing any overlap.

          * LIST-Any buffers sharing attribute values in the listed fields
          (carried over from the input features) are dissolved.
      dissolve_fields {Field}:
          A list of one or more fields from the input features on which output
          buffers will be dissolved. Any buffers sharing attribute values in the
          listed fields are dissolved. This option is only required when the
          dissolve_option is LIST.
      summary_fields {Value Table}:
          Statistics that may be applied to numeric and string fields. If left
          empty, only total count will be calculated. These statistics are only
          applied when the dissolve_option is LIST or ALL.

          * COUNT-Calculates the number of nonnull values. Can be used on
          numeric fields or strings. The count of [null, 0, 2] is 2.

          * SUM-The sum of numeric values in a field. The sum of [null, null, 3]
          is 3.

          * MEAN-The mean of numeric values. The mean of [0, 2, null] is 1.

          * MIN-The minimum value of a numeric field. The minimum of [0, 2,
          null] is 0.

          * MAX-The maximum value of a numeric field. The maximum value of [0,
          2, null] is 2.

          * STDDEV-The standard deviation of a numeric field. The standard
          deviation of [1] is null. The standard deviation of [null, 1,1,1] is
          1.

          * VAR-The variance of a numeric field in a track. The variance of [1]
          is null. The variance of [null, 1, 1, 1] is 1.

          * RANGE-The range of a numeric field. This is calculated as the
          minimum values subtracted form the maximum value. The range of [0,
          null, 1] is 1. The range of [null, 4] is 0.

          * ANY-This is a sample string from a field of type string.
      multipart {Boolean}:
          Specifies whether multipart features can be created.

          * MULTI_PART-Output multipart features will be created where
          appropriate.

          * SINGLE_PART-Instead of creating multipart features, individual
          features will be created for each part. This is the default.
      data_store {String}:
          The ArcGIS Data Store to which the output will be saved. The default
          is SPATIOTEMPORAL_DATA_STORE. All results stored to the
          SPATIOTEMPORAL_DATA_STORE will be stored in WGS84. Results stored in a
          RELATIONAL_DATA_STORE will maintain their coordinate system.

          * SPATIOTEMPORAL_DATA_STORE-Output will be stored in your
          spatiotemporal big data store. This is the default.

          * RELATIONAL_DATA_STORE-Output will be stored in your relational data
          store."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateBuffers_geoanalytics(*gp_fixargs((input_layer, output_name, method, buffer_type, buffer_field, buffer_distance, buffer_expression, dissolve_option, dissolve_fields, summary_fields, multipart, data_store), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject